# -*- coding: utf-8 -*-
"""
Overview
=======
 A collection of assets commonly needed by disparate components of the
 Quickshot client.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging
import os

import gtk

_DATA_PATH = '../data/' #: The relative path to Quickshot's data.

QS_PATH = os.path.expanduser('~/.quickshot') #: The path of the user's Quickshot operation data.
_QS_CONTRIBUTOR_PATH = '/contributor' #: The sub-path to the user's contributor details.

logger = logging.getLogger('quickshot.common')

def get_data_file(*path_segments):
    """
    Provides the absolute path of the requested Quickshot data-path.

    Example: get_data_file('media', 'hello.png') ->
    '/home/flan/quickshot/data/media/hello.png'

    @type path_segments: basestr*
    @param path_segments: A series of sub-path elements relative to Quickshot's
        data-path.

    @rtype: basestr
    @return: The absolute path to the requested location.

    @raise DataPathNotFoundError: The requested location could not be found.
    """
    abs_data_path = os.path.join(get_data_path(), *path_segments)
    if os.path.exists(abs_data_path):
        return abs_data_path
    else:
        raise DataPathNotFoundError("'%(path)s' does not exist" % {
         'path': abs_data_path,
        })
        
def get_data_path():
    """
    Provides the absolute path of Quickshot's root data-path.

    @rtype: basestr
    @return: The absolute path to Quickshot's root data-path.

    @raise DataPathNotFoundError: The root data-path could not be found.
    """
    abs_data_path = os.path.abspath(os.path.dirname(__file__) + '/' + _DATA_PATH)
    if os.path.exists(abs_data_path):
        return abs_data_path
    else:
        raise DataPathNotFoundError("'%(path)s' does not exist" % {
         'path': abs_data_path,
        })
        
def get_pixbuf_from_string(data, content_type):
    """
    Builds a GTK-compatible Pixbuf from a raw data string.

    @type data: str
    @param data: Raw image data.
    @type content_type: str
    @param content_type: The data's format-indicator, like 'png' or 'jpeg'.

    @rtype: C{gtk.gdk.Pixbuf}
    @return: The assembled Pixbuf.
    """
    pixbuf_loader = gtk.gdk.PixbufLoader(image_type=content_type)
    pixbuf_loader.write(data)
    pixbuf_loader.close()
    
    return pixbuf_loader.get_pixbuf()

def read_contributor_details():
    """
    Provides the user's contributor information.

    @rtype: dict | None
    @return: The user's contributor information, a dict of the form
        {
         'e-mail':basestr,
         'name': basestr|None,
         'homepage': basestr|None,
         'publish': bool,
        }
        or None if the user's contributor details are undefined or invalid.
    """
    if not os.path.isfile(QS_PATH + _QS_CONTRIBUTOR_PATH):
        return None
        
    details = {}
    try:
        contributor = open(QS_PATH + _QS_CONTRIBUTOR_PATH, 'r')
        details['e-mail'] = contributor.readline().strip() or None
        details['name'] = contributor.readline().strip() or None
        details['homepage'] = contributor.readline().strip() or None
        details['publish'] = contributor.readline().strip().lower() == 'true'
    except Exception, e:
        logger.warn("Unable to read contributor details: %(error)s" % {
         'error': str(e),
        })
        return None
    else:
        contributor.close()
        
        if not details.get('e-mail'):
            return None
        return details
        
def write_contributor_details(email, name, homepage, publish):
    """
    Sets the user's contributor information.

    @type email: basestr
    @param email: The user's e-mail address.
    @type name: basestr | None
    @param name: The user's name.
    @type homepage: basestr | None
    @param homepage: The user's homepage.
    @type publish: bool
    @param publish: True if the user's details should be published.
    
    @rtype: bool
    @return: True if the user's contributor details were successfully updated.
    """
    try:
        contributor = open(QS_PATH + _QS_CONTRIBUTOR_PATH, 'w')
        contributor.write(email + '\n')
        contributor.write((name or '') + '\n')
        contributor.write((homepage or '') + '\n')
        contributor.write(str(publish) + '\n')
    except Exception, e:
        logger.error("Unable to write contributor details: %(error)s" % {
         'error': str(e),
        })
        return False
    else:
        contributor.close()
        return True
        
class Error(Exception):
    """
    The base class from which all exceptions native to this module inherit.
    """
    pass
    
class DataPathNotFoundError(Error):
    """
    Signifies that the requested resource could not be found.
    """
    pass
    
