# -*- coding: utf-8 -*-
"""
Overview
=======
 Provides a general convenience template for all Quickshot client windows that
 must be presented as master windows in the normal flow of execution.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import webbrowser

import gtk

import about
import user_details

_LAUNCHPAD_NAME = 'quickshot' #: The name of the Quickshot package in Launchpad.
_URL_HELP = "http://quickshot.org/" #: The project website URL.
_URL_BUGS = "https://bugs.launchpad.net/" + _LAUNCHPAD_NAME #: The bug-feed URL
_URL_TRANSLATIONS = "https://translations.launchpad.net/" + _LAUNCHPAD_NAME #: The translation URL.

PROGRAM_NAME = 'Quickshot' #The name of the project to be displayed to users.
PROGRAM_NAME_SUFFIX = ' - ' + PROGRAM_NAME #: The suffix to apply to titles.

class BaseWindow(gtk.Window):
    """
    A template class used to ensure that a consistent menubar and statusbar
    are applied to all key windows in the system.
    """
    _sbr_status = None #: The status-bar for this window.
    
    def __init__(self, title=None):
        """
        Initialises the menu bar and loads the core window content into the
        centre of a new window.

        @type title: basestring|None
        @param title: The title of the window.

            Default: None
        """
        gtk.Window.__init__(self)
        
        self.set_title(title)
        acceleration_group = gtk.AccelGroup()
        self.add_accel_group(acceleration_group)
        
        mbr_menu_bar = gtk.MenuBar()
        
        #File menu.
        mit_file = gtk.MenuItem("_File")
        mnu_file = gtk.Menu()
        mit_file.set_submenu(mnu_file)
        
        mit_file_quit = gtk.ImageMenuItem(gtk.STOCK_QUIT, acceleration_group)
        (key, modifiers) = gtk.accelerator_parse("Q")
        mit_file_quit.add_accelerator("activate", acceleration_group, key, modifiers, gtk.ACCEL_VISIBLE)
        mit_file_quit.connect("activate", gtk.main_quit)
        mnu_file.append(mit_file_quit)
        
        mbr_menu_bar.append(mit_file)
        
        #Edit menu.
        mit_edit = gtk.MenuItem("_Edit")
        mnu_edit = gtk.Menu()
        mit_edit.set_submenu(mnu_edit)
        
        mit_edit_credit = gtk.MenuItem("User credit")
        mit_edit_credit.connect("activate", self._show_contributor_details)
        mnu_edit.append(mit_edit_credit)
        
        mbr_menu_bar.append(mit_edit)
        
        #Help menu.
        mit_help = gtk.MenuItem("_Help")
        mnu_help = gtk.Menu()
        mit_help.set_submenu(mnu_help)
        
        try: #Attempt to integrate with Launchpad.
            import LaunchpadIntegration
            LaunchpadIntegration.set_sourcepackagename(_LAUNCHPAD_NAME)
            LaunchpadIntegration.add_items(mnu_help, 0, False, True)
        except ImportError:
            mit_help_online = gtk.MenuItem("Get Help Online...")
            mit_help_online.connect("activate", self._show_url, _URL_HELP)
            mnu_help.append(mit_help_online)
            
            mit_help_translate = gtk.MenuItem("Translate This Application...")
            mit_help_translate.connect("activate", self._show_url, _URL_TRANSLATIONS)
            mnu_help.append(mit_help_translate)

            mit_help_bug = gtk.MenuItem("Report a Problem...")
            mit_help_bug.connect("activate", self._show_url, _URL_BUGS)
            mnu_help.append(mit_help_bug)
            
            mit_help__separator_1 = gtk.SeparatorMenuItem()
            mnu_help.append(mit_help__separator_1)
            
        mit_help_about = gtk.MenuItem("About")
        mit_help_about.connect("activate", self._show_about)
        mnu_help.append(mit_help_about)
        
        mbr_menu_bar.append(mit_help)
        
        box_main = gtk.VBox(False, 3)
        self._sbr_status = gtk.Statusbar()
        box_main.pack_start(mbr_menu_bar, False, True, 0)
        box_main.pack_start(self._build_content(), True, True, 0)
        box_main.pack_start(self._sbr_status, False, True, 0)
        
        self.add(box_main)
        
        self._set_bindings()
        
        self._set_properties()
        
        self.show_all()
        
    def set_title(self, title):
        """
        Sets the title for this window.

        @type title: basestr | None
        @param title: The title of this window.
        """
        if title:
            gtk.Window.set_title(self, title + PROGRAM_NAME_SUFFIX)
        else:
            gtk.Window.set_title(self, PROGRAM_NAME)

    def _build_content(self):
        """
        Sets up the main body of the window.

        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        return gtk.Label("Inherit and override me!")
       
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        pass
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        pass
        
    def _show_about(self, *args):
        """
        Displays the about window.

        @type *args: arg*
        @param *args: Not used.
        """
        about.AboutWindow(self)

    def _show_contributor_details(self, *args):
        """
        Displays the contributor-details window.

        @type *args: arg*
        @param *args: Not used.
        """
        user_details.UserDetailsWindow(self)
        
    def _show_url(self, gtk_object, url):
        """
        Opens a URL in the user's default web browser.

        @type gtk_object: gtk.Widget
        @param gtk_object: Not used.
        @type url: basestr
        @param url: The URL to open.
        """
        webbrowser.open(url)
        
