# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the process of displaying and processing dialogue boxes in the
 Quickshot client.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import gtk
import pango

import base

def _present_dialogue(dialogue, icon, details):
    """
    Assembles and presents a dialogue box to the user.

    @type dialogue: gtk.Dialog
    @param dialogue: The dialogue box to be populated and displayed.
    @type icon: int
    @param icon: The GTK stock-icon-constant representing the image to be
        shown.
    @type details: sequence
    @param details: A collection of basestrings to use as the message's body.

    @rtype: int
    @return: The user's selection, based on the button clicked.
    """
    box_summary = gtk.HBox()
    
    img = gtk.Image()
    img.set_from_stock(icon, gtk.ICON_SIZE_DIALOG)
    box_summary.pack_start(img, False, True, 0)
    
    box_details = gtk.VBox()
    for detail in details:
        lbl_detail = gtk.Label(detail)
        lbl_detail.set_alignment(0.0, 0.5)
        lbl_detail.set_width_chars(60)
        lbl_detail.set_line_wrap(True)
        lbl_detail.set_line_wrap_mode(pango.WRAP_WORD)
        lbl_detail.set_selectable(True)
        box_details.pack_start(lbl_detail, True, True, 0)
    box_summary.pack_start(box_details, True, True, 0)
    
    dialogue.vbox.pack_start(box_summary, True, True, 0)
    box_summary.show_all()
    
    dialogue.set_resizable(False)
    dialogue.set_keep_above(True)
    
    result = dialogue.run()
    dialogue.destroy()
    return result
    
def error(parent, details, recoverable=True):
    """
    Assembles and presents an error dialogue box to the user.

    @type parent: gtk.Window
    @param parent: The window that will serve as this box's parent.
    @type details: sequence
    @param details: A collection of basestrings to use as the message's body.
    @type recoverable: bool
    @param recoverable: True if the user should be given an 'OK' button
        (C{gtk.RESPONSE_YES}) to continue the current operation.

        Default: True.

    @rtype: int
    @return: The user's selection, based on the button clicked.
        If not recoverable, this is always C{gtk.RESPONSE_NO}.
    """
    buttons = None
    if recoverable:
        buttons = (
         gtk.STOCK_STOP, gtk.RESPONSE_NO,
         gtk.STOCK_OK, gtk.RESPONSE_YES,
        )
    else:
        buttons = (
         gtk.STOCK_STOP, gtk.RESPONSE_NO,
        )
        
    dialogue = gtk.Dialog(
     title=('Error' + base.PROGRAM_NAME_SUFFIX),
     parent=parent,
     flags=(gtk.DIALOG_MODAL),
     buttons=buttons
    )

    return _present_dialogue(dialogue, gtk.STOCK_DIALOG_ERROR, details)
    
def info(parent, details):
    """
    Assembles and presents an info dialogue box to the user.

    @type parent: gtk.Window
    @param parent: The window that will serve as this box's parent.
    @type details: sequence
    @param details: A collection of basestrings to use as the message's body.
    
    @rtype: int
    @return: The user's selection, based on the button clicked.
        This is always C{gtk.RESPONSE_YES}.
    """
    dialogue = gtk.Dialog(
     title=('Info' + base.PROGRAM_NAME_SUFFIX),
     parent=parent,
     flags=(gtk.DIALOG_MODAL),
     buttons=(
      gtk.STOCK_OK, gtk.RESPONSE_YES,
     )
    )
    
    return _present_dialogue(dialogue, gtk.STOCK_DIALOG_INFO, details)
    
def question(parent, details):
    """
    Assembles and presents a question dialogue box to the user.

    @type parent: gtk.Window
    @param parent: The window that will serve as this box's parent.
    @type details: sequence
    @param details: A collection of basestrings to use as the message's body.
    @type abortable: bool
    
    @rtype: int
    @return: The user's selection, based on the button clicked.
        This is always one of (C{gtk.RESPONSE_NO}, C{gtk.RESPONSE_YES}).
    """
    dialogue = gtk.Dialog(
     title=('Question' + base.PROGRAM_NAME_SUFFIX),
     parent=parent,
     flags=(gtk.DIALOG_MODAL),
     buttons=(
      gtk.STOCK_NO, gtk.RESPONSE_NO,
      gtk.STOCK_YES, gtk.RESPONSE_YES,
     )
    )
    
    return _present_dialogue(dialogue, gtk.STOCK_DIALOG_QUESTION, details)
    
def warn(parent, details, abortable=False):
    """
    Assembles and presents a warning dialogue box to the user.

    @type parent: gtk.Window
    @param parent: The window that will serve as this box's parent.
    @type details: sequence
    @param details: A collection of basestrings to use as the message's body.
    @type abortable: bool
    @param abortable: True if the user should be given a 'Stop' button
        (C{gtk.RESPONSE_NO}) to abort the current operation.

        Default: False.

    @rtype: int
    @return: The user's selection, based on the button clicked.
        If not abortable, this is always C{gtk.RESPONSE_YES}.
    """
    buttons = None
    if abortable:
        buttons = (
         gtk.STOCK_STOP, gtk.RESPONSE_NO,
         gtk.STOCK_OK, gtk.RESPONSE_YES,
        )
    else:
        buttons = (
         gtk.STOCK_OK, gtk.RESPONSE_YES,
        )
        
    dialogue = gtk.Dialog(
     title=('Warning' + base.PROGRAM_NAME_SUFFIX),
     parent=parent,
     flags=(gtk.DIALOG_MODAL),
     buttons=buttons
    )
    
    return _present_dialogue(dialogue, gtk.STOCK_DIALOG_WARNING, details)
    
