# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's resolution-changing
 window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging

import gtk

import quickshot.common
import quickshot.lib.resolution

import base
import graphics_confirm
import screenshot_list
import dialogue

logger = logging.getLogger('quickshot.gui.graphics_change')

_RESOLUTION_TIMEOUT = 30 #: The number of seconds to allow before resetting the user's display.

class GraphicsChangeWindow(base.BaseWindow):
    """
    A window used to inform the user that their screen resolution needs to be
    changed.
    """
    _session = None #: A L{quickshot.lib.server._Session} that manages communication with the server.
    _details = None #: A dictionary of project details.
    _resolution_x = None #: The target X-resolution.
    _resolution_y = None #: The target Y-resolution.
    _old_resolution_x = None #: The pre-existing X-resolution.
    _old_resolution_y = None #: The pre-existing Y-resolution.
    _btn_forward = None #: A button used to engage the resolution-change.
    _btn_quit = None #: A button used to quit Quickshot.
    _sig_destroy = None #: A signal-binding used to handle the window's on-destroy event.
    
    def __init__(self, session, details):
        """
        Either creates a graphics-change window or bypasses this step, immediately
        loading the screenshot-list window.

        The decision is made based on whether the current resolution meets
        project-required criteria.

        @type session: L{quickshot.lib.server._Session}
        @param session: A session object for managing communication with the
            project server.
        @type details: dict
        @param details: A collection of project details.
        """
        self._session = session
        self._details = details
        self._resolution_x = details['config']['resolution']['x']
        self._resolution_y = details['config']['resolution']['y']
        (self._old_resolution_x, self._old_resolution_y) = quickshot.lib.resolution.get_resolution()
        
        if (
         not (self._resolution_x and not self._resolution_x == self._old_resolution_x)
         and not (self._resolution_y and not self._resolution_y == self._old_resolution_y)
        ):
            logger.debug("Screen resolution already compatible with project")
            screenshot_list.ScreenshotListWindow(session, details)
            return

        #If the resolution is undefined for one value (legal in the server, but
        #ambiguous in current rasterising logic), assume 4:3.
        if self._resolution_x and not self._resolution_y:
            self._resolution_y = int(self._resolution_x * (3.0 / 4.0))
        elif self._resolution_y and not self._resolution_x:
            self._resolution_x = int(self._resolution_y * (4.0 / 3.0))
            
        base.BaseWindow.__init__(self)
        
    def _build_content(self):
        """
        Loads the window structure from Glade, then sets static values in the
        labels.

        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        logger.debug("Loading graphics_change")
        
        builder = gtk.Builder()
        builder.add_from_file(quickshot.common.get_data_file('ui', 'graphics_change.ui'))
        
        lbl_summary = builder.get_object('lbl_summary')
        lbl_summary.set_markup(lbl_summary.get_text() % {
         'version_name': '<i>' + self._details['meta']['version']['friendly-name'] + '</i>',
         'resolution_x': self._resolution_x,
         'resolution_y': self._resolution_y,
         'timeout': _RESOLUTION_TIMEOUT,
        })
        
        self._btn_quit = builder.get_object('btn_quit')
        self._btn_forward = builder.get_object('btn_forward')
        
        content = builder.get_object('box_main')
        self.set_title(content.parent.get_title())
        self.set_size_request(*content.parent.get_default_size())
        self.set_resizable(content.parent.get_resizable())
        self._sbr_status.set_has_resize_grip(self.get_resizable())
        self.set_icon(content.parent.get_icon())
        content.unparent()
        return content

    def _change_resolution(self, *args):
        """
        Changes the host's screen resolution, warning the user if it failed.

        L{_resolution_change_success} and L{_resolution_change_failure} are
        passed to the confirmation window, displayed on an apparently
        successful change; these callbacks are used to get user approval of the
        modifications.
        
        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("User agreed to change resolution")
        
        context_id = self._sbr_status.get_context_id('change resolution')
        self._sbr_status.push(context_id, "Changing screen resolution...")
        try:
            quickshot.lib.resolution.set_resolution(self._resolution_x, self._resolution_y)
        except Exception, e:
            logger.error("Unable to change screen resolution: %(error)s" % {
             'error': str(e),
            })
            dialogue.error(self, (
             "Unable to change screen resolution",
             "Error: " + str(e),
            ), recoverable=False)
            gtk.main_quit()
        else:
            graphics_confirm.GraphicsConfirmWindow(
             self, _RESOLUTION_TIMEOUT,
             self._resolution_change_failure, self._resolution_change_success
            )
        finally:
            self._sbr_status.pop(context_id)
            
    def _resolution_change_failure(self):
        """
        Rolls the screen resolution back to its prior state.
        """
        logger.info("Restoring previous resolution")
        
        context_id = self._sbr_status.get_context_id('change resolution')
        self._sbr_status.push(context_id, "Changing screen resolution...")
        try:
            quickshot.lib.resolution.set_resolution(self._old_resolution_x, self._old_resolution_y)
        except Exception, e:
            logger.error("Unable to restore screen resolution: %(error)s" % {
             'error': str(e),
            })
            dialogue.error(self, (
             "Unable to restore screen resolution",
             "Error: " + str(e),
            ), recoverable=False)
            gtk.main_quit()
        finally:
            self._sbr_status.pop(context_id)

    def _resolution_change_success(self):
        """
        Leaves the screen resolution as is and advances to the screenshot-list.
        """
        logger.info("User indicated that new resolution works")
        
        self.hide()
        
        screenshot_list.ScreenshotListWindow(self._session, self._details)
        
        self.disconnect(self._sig_destroy)
        self.destroy()
        
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        self._sig_destroy = self.connect("destroy", gtk.main_quit)
        
        self._btn_quit.connect("clicked", gtk.main_quit)
        self._btn_forward.connect("clicked", self._change_resolution)
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        self.set_position(gtk.WIN_POS_CENTER)
        
