# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's
 resolution-change-confirming window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging

import gobject
import gtk

import quickshot.common

logger = logging.getLogger('quickshot.gui.graphics_confirm')

def GraphicsConfirmWindow(parent, timeout, revert_callback, accept_callback):
    """
    Sets up and shows the graphics-confirm-window.

    @type parent: gtk.Window
    @param parent: The window to use as a parent.
    @type timeout: int
    @param timeout: The number of seconds to wait for before assuming the
        change failed.
    @type revert_callback: callable
    @param revert_callback: A function that takes no arguments, indicating that
        the update failed.
    @type accept_callback: callable
    @param accept_callback: A function that takes no arguments, indicatind that
        the update succeeded.

    @rtype: gtk.Window
    @return: The graphics-confirm-window.
    """
    logger.debug("Loading graphics_confirm")
    
    builder = gtk.Builder()
    builder.add_from_file(quickshot.common.get_data_file('ui', 'graphics_confirm.ui'))
    
    graphics_confirm_window = builder.get_object('win_graphics_confirm')
    graphics_confirm_window.connect("destroy", _callback, revert_callback, graphics_confirm_window)
    graphics_confirm_window.set_transient_for(parent)
    graphics_confirm_window.set_keep_above(True)

    btn_accept = builder.get_object('btn_accept')
    btn_accept.connect("clicked", _callback, accept_callback, graphics_confirm_window)
    
    btn_revert = builder.get_object('btn_revert')
    btn_revert.connect("clicked", _callback, revert_callback, graphics_confirm_window)
    
    graphics_confirm_window.is_alive = True
    
    countdown_updater = _update_countdown(
     timeout, builder.get_object('lbl_countdown'),
     revert_callback, graphics_confirm_window
    )
    if countdown_updater.next():
        gobject.timeout_add(1000, countdown_updater.next)
    else:
        logger.critical("No resolution-change window offered to user")
        revert_callback()
        
    graphics_confirm_window.show()
    return graphics_confirm_window
    
def _callback(button, callback, window):
    """
    Issues a callback and terminates the countdown, simultaneously destroying
    the confirm-window.

    Calling this function multiple times is safe, as it becomes a no-op.

    @type button: gtk.Button
    @param button: Not used.
    @type callback: callable
    @param callback: A function that takes no arguments, indicating the result
        of the update.
    @type window: gtk.Window
    @param window: The confirm-window, augmented with an C{is_alive:bool}
        attribute.
    """
    if window.is_alive:
        logger.debug("Sending callback response")
        window.is_alive = False
        window.destroy()
        callback()
        
def _update_countdown(timeout, label, callback, window):
    """
    A generator function that updates the displayed countdown, decrementing it
    by one second each time, until the timeout hits 0.

    When the timeout hits zero, the rejection callback is fired.

    @type timeout: int
    @param timeout: The number of seconds to wait before assuming failure.
    @type label: gtk.Label
    @param label: The label to be updated.
    @type callback: callable
    @param callback: A function that takes no arguments, indicating that the
        update failed.
    @type window: gtk.Window
    @param window: The confirm-window, augmented with an C{is_alive:bool}
        attribute.

    @rtype: bool
    @return: True if the countdown is ongoing.
    """
    text = label.get_label()
    while timeout:
        if window.is_alive:
            logger.debug("Updating resolution-change timeout: " + str(timeout))
            
            label.set_label(text % {
             'count': str(timeout) + ' second' + ((not timeout == 1 and 's') or ''),
            })
            timeout -= 1
            
            yield True
    if window.is_alive:
        _callback(None, callback, window)
    yield False
    
