# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's
 project-selection/welcome window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging
import os
import urllib

import gio
import gobject
import gtk

import quickshot.common
import quickshot.lib.server

import base
import user_account
import screenshot_list
import dialogue

logger = logging.getLogger('quickshot.gui.project_selection')

_RECENT_PROJECTS_FILE = quickshot.common.QS_PATH + '/recent' #The absolute path to the recent projects file.
_RECENT_PROJECTS_COUNT = 10 #: The number of entries to keep in the recent projects file.

class ProjectSelectionWindow(base.BaseWindow):
    """
    A window used to greet the user and allow them to select the project they
    wish to work on.
    """
    _btn_go_recent = None #: A button used to activate a project in the recent projects list.
    _btn_go_public = None #: A button used to activate a project in the public projects list.
    _btn_go_local = None #: A button used to activate a project loaded from a local .qsproj file.
    _btn_go_url = None #: A button used to activate a project specified as a URL.
    _lst_recent = None #: A list of all recent projects.
    _lst_public = None #: A list of all public projects.
    _file_local = None #: A selector for local .qsproj files.
    _txt_url = None #: An input for direct project URLs.
    _sig_destroy = None #: A signal-binding used to handle the window's on-destroy event.
    
    def __init__(self, qsproj=None, url=None):
        """
        Creates a project-selection window to greet the user.

        If a filename or URL was given as an argument, the appropriate field is
        pre-populated.

        @type qsproj: basestring | None
        @param qsproj: The path to a .qsproj file.
        @type url: basestring | None
        @param url: A project URL.
        """
        base.BaseWindow.__init__(self)
        
        self._populate_recent()
        self._populate_public()
        
        if qsproj:
            gio_path = os.path.expanduser(qsproj)
            self._file_local.select_file(gio.File(gio_path))
            
        if url:
            self._txt_url.set_text(url)
            
    def _build_content(self):
        """
        Loads the window structure from Glade.

        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        logger.debug("Loading project_selection")
        
        builder = gtk.Builder()
        builder.add_from_file(quickshot.common.get_data_file('ui', 'project_selection.ui'))
        
        self._lst_recent = builder.get_object('lst_recent')
        self._btn_go_recent = builder.get_object('btn_go_recent')
        
        self._lst_public = builder.get_object('lst_public')
        self._btn_go_public = builder.get_object('btn_go_public')
        
        self._file_local = builder.get_object('file_local')
        self._btn_go_local = builder.get_object('btn_go_local')
        
        self._txt_url = builder.get_object('txt_url')
        self._btn_go_url = builder.get_object('btn_go_url')
        
        content = builder.get_object('box_main')
        self.set_title(content.parent.get_title())
        self.set_size_request(*content.parent.get_default_size())
        self.set_resizable(content.parent.get_resizable())
        self._sbr_status.set_has_resize_grip(self.get_resizable())
        self.set_icon(content.parent.get_icon())
        content.unparent()
        return content

    def _connect_to_server(self, url):
        """
        Connects to a Quickshot server.

        Upon success, the recent projects list is updated and control moves to
        the user-account window.

        @type url: basestring
        @param url: The URL to which a connection should be established.
        """
        logger.debug("Downloading project details from " + url)
        
        context_id = self._sbr_status.get_context_id('download qsproj')
        self._sbr_status.push(context_id, "Downloading project data from server...")
        try:
            (session, details) = quickshot.lib.server.build_session_from_url(
             url, os.getenv('LANG')[:os.getenv('LANG').find('.')]
            )
        except quickshot.lib.server.CommunicationError, e:
            logger.error("Unable to downlod project details from %(url)s: %(error)s" % {
             'url': url,
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to download project details from %(url)s" % {
              'url': url,
             },
             "Error: " + str(e),
            ))
            self._sbr_status.pop(context_id)
        else:
            self.hide()
            
            user_account.UserAccountWindow(session, details)
            
            self._update_recent_projects(details, url)
            
            self.disconnect(self._sig_destroy)
            self.destroy()
            
    def _file_changed(self, file_chooser, button):
        """
        Updates the file-activation path when a file is selected.

        @type file_chooser: gtk.FileSelector
        @param file_chooser: The selector that activated.
        @type button: gtk.Button
        @param button: The button tied to the selector.
        """
        logger.debug("New file chosen")
        button.set_sensitive(bool(file_chooser.get_uri()))

    def _go_local(self, *args):
        """
        Parses a local .qsproj file and connects to the project URL.

        @type *args: arg*
        @param *args: Not used.
        """
        uri = self._file_local.get_uri()
        qsproj = urllib.unquote(uri[uri.find('//') + 2:])
        
        logger.debug("Reading project file " + qsproj)
        try:
            url = quickshot.lib.server.get_url_from_qsproj(qsproj)
        except IOError, e:
            logger.error("Unable to read from local file, %(file)s: %(error)s" % {
             'file': qsproj,
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to read from project file, %(file)s" % {
              'file': qsproj,
             },
             "Error: " + str(e),
            ))
        else:
            self._connect_to_server(url)
            
    def _go_recent(self, *args):
        """
        Connects to the project URL associated with the selected recent project
        entry, if any.

        @type *args: arg*
        @param *args: Not used.
        """
        selected = self._lst_recent.get_active_iter()
        if selected:
            model = self._lst_recent.get_model()
            url = model.get_value(selected, 1)
            self._connect_to_server(url)
            
    def _go_public(self, *args):
        """
        Connects to the project URL associated with the selected public project
        entry, if any.

        @type *args: arg*
        @param *args: Not used.
        """
        """
        Implementation plan:
	        Upload a .qsproj to the site with a mandatory short summary and
	        "about this project" URL.
		        This information gets added to the .qsproj recorded in
		        the site's filesystem (some sort of [_special_] section)
		
	        The client, upon requesting a list, gets every active project's
	        friendly names, URL, summary, and about-URL.
	
	        After a friendly-name-pair is chosen, the summary and about-URL
	        are shown to the user in a modal pop-up, who must choose to
	        continue or go back to the selection screen.

	        Site logic: Name local .qsproj files after the project URL
	        contained therein (http://flan.uguu.ca:5000/ump/10-04 ->
	        flan.uguu.ca:5000_ump_10-04 -- discard any path information in
	        the middle)
	
	        Allow .qsproj files to be replaced by anyone (it's a trusting
	        bulletin board model)
	
	        When scanning local .qsproj files to build the data to send to
	        the client, just delete anything that's expired, to make
	        future passes faster.
	
	        Log every upload by IP, in case someone needs to be
	        blacklisted.
        """
        pass
        
    def _go_url(self, *args):
        """
        Connects to the given project URL.

        @type *args: arg*
        @param *args: Not used.
        """
        url = self._txt_url.get_text()
        self._connect_to_server(url)

    def _populate_public(self):
        """
        Queries the public projects server and inserts entries into the public
        projects list, selecting the first item on completion.
        """
        pass
        
    def _populate_recent(self):
        """
        Loads the recent projects data and inserts entries into the recent
        projects list, selecting the first item on completion.
        """
        logger.debug("Loading recent projects")
        
        recent = gtk.ListStore(
         gobject.TYPE_STRING,
         gobject.TYPE_STRING
        )
        
        context_id = self._sbr_status.get_context_id('recent projects')
        self._sbr_status.push(context_id, "Looking for recently accessed projects...")
        recent_projects = None
        try:
            recent_projects = self._read_recent_projects()
        except Exception, e:
            logger.warn("Unable to read from %(recent_projects)s: %(error)s" % {
             'recent_projects': _RECENT_PROJECTS_FILE,
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to read recent projects file, %(file)s" % {
              'file': _RECENT_PROJECTS_FILE,
             },
             "Error: " + str(e),
            ))
        else:
            for (name, url) in recent_projects:
                recent.append((name, url))
        finally:
            self._sbr_status.pop(context_id)
            
        self._lst_recent.set_model(recent)
        cell = gtk.CellRendererText()
        self._lst_recent.pack_start(cell, True)
        self._lst_recent.add_attribute(cell, 'text', 0)
        
        if recent_projects:
            self._lst_recent.set_active(0)
            self._btn_go_recent.set_sensitive(True)

    def _read_recent_projects(self):
        """
        Builds a list of all recent projects.

        @rtype: list
        @return: A list of tuples containing the project's name and its URL.
        """
        recent_projects = []
        if os.path.isfile(_RECENT_PROJECTS_FILE):
            recent_projects_file = open(_RECENT_PROJECTS_FILE, 'r')
            for line in recent_projects_file.readlines():
                if line:
                    parts = line.split('\t', 1)
                    if len(parts) == 2:
                        recent_projects.append((parts[0], parts[1].strip()))
            recent_projects_file.close()
        return recent_projects
        
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        self._sig_destroy = self.connect("destroy", gtk.main_quit)
        
        self._file_local.connect("selection-changed", self._file_changed, self._btn_go_local)
        self._txt_url.connect("changed", self._url_changed, self._btn_go_url)
        
        self._btn_go_recent.connect("clicked", self._go_recent)
        self._btn_go_public.connect("clicked", self._go_public)
        self._btn_go_local.connect("clicked", self._go_local)
        self._btn_go_url.connect("clicked", self._go_url)
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        self.set_position(gtk.WIN_POS_CENTER)
        
        self._btn_go_recent.set_sensitive(False)
        self._btn_go_public.set_sensitive(False)
        self._btn_go_local.set_sensitive(False)
        self._btn_go_url.set_sensitive(False)
        
        qsproj_filter = gtk.FileFilter()
        qsproj_filter.set_name('Quickshot projects')
        qsproj_filter.add_pattern('*.qsproj')
        self._file_local.add_filter(qsproj_filter)
        
    def _update_recent_projects(self, details, url):
        """
        Writes a record of all recent projects.

        The new record is inserted at the top, formed from a combination of
        project and version friendly names and the project URL.

        If the URL already existed later in the list, it is not repeated.

        Only up to C{_RECENT_PROJECTS_COUNT} items are recorded.

        @type details: dict
        @param details: A collection of project details from the server.
        @type url: basestring
        @param url: The URL of the project.
        """
        logger.debug("Updating recent projects")
        
        context_id = self._sbr_status.get_context_id('recent projects')
        self._sbr_status.push(context_id, "Updating recently accessed projects...")
        try:
            recent_projects = [(details['meta']['project']['friendly-name'] + ' / ' + details['meta']['version']['friendly-name'], url)]
            recent_projects += [(recent_name, recent_url) for (recent_name, recent_url) in self._read_recent_projects() if not recent_url == url]
        except Exception, e:
            logger.warn("Unable to write to %(recent_projects)s: %(error)s" % {
             'recent_projects': _RECENT_PROJECTS_FILE,
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to update recent projects file, " + _RECENT_PROJECTS_FILE,
             "Error: " + str(e),
            ))
        else:
            recent_projects_file = open(_RECENT_PROJECTS_FILE, 'w')
            for (name, url) in recent_projects[:_RECENT_PROJECTS_COUNT]:
                recent_projects_file.write(name + '\t' + url + '\n')
            recent_projects_file.close()
        finally:
            self._sbr_status.pop(context_id)
            
    def _url_changed(self, entry, button):
        """
        Updates the URL-activation path when text is entered into the URL
        field.

        @type entry: gtk.Entry
        @param entry: The field that activated.
        @type button: gtk.Button
        @param button: The button tied to the field.
        """
        button.set_sensitive(bool(entry.get_text()))
        
