# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's
 screenshot-comparison/pre-upload window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging

import gtk

import quickshot.common

import base
import screenshot_detail
import dialogue

logger = logging.getLogger('quickshot.gui.screenshot_compare')

class ScreenshotCompareWindow(base.BaseWindow):
    """
    A window used to present the user with an opportunity to confirm that the
    screenshot they have taken matches with their expectations.
    """
    _session = None #: A L{quickshot.lib.server._Session} that manages communication with the server.
    _screenshot_name = None #: The system-name of the screenshot being worked on.
    _captured_screenshot = None #: A byte-string and format-code containing the captured screenshot.
    _screenshot_pending = None #: True if the screenshot is in a pending state on the server.
    _screenshot_details = None #: Details about the screenshot to be captured.
    _screenshot_list_window = None #: A reference to the screenshot-list window.
    _reference_thumbnail = None #: A byte-string and format-code containing the reference screenshot in thumbnail form.
    _reference_screenshot = None #: A byte-string and format-code containing the reference screenshot.

    def __init__(self,
     session, screenshot_name, captured_screenshot, screenshot_details, screenshot_pending, screenshot_list_window,
     _reference_thumbnail=None, _reference_screenshot=None
    ):
        """
        Creates a screenshot-compare window, used to allow the user to review
        their captured screenshot prior to uploading.

        The reference screenshot, if not previously cached as part of a
        recapture loop, is downloaded from the server, if it is available,
        and displayed alongside the captured screenshot, for the user to
        visually screen before uploading it to the server.
        
        @type session: L{quickshot.lib.server._Session}
        @param session: A session object for managing communication with the
            project server.
        @type screenshot_name: basestring
        @param screenshot_name: The system-name of the screenshot.
        @type captured_screenshot: tuple(2)
        @param captured_screenshot: A byte-string and format-code containing
            the captured screenshot.
        @type screenshot_details: dict
        @param screenshot_details: Details about the screenshot that was
            captured.
        @type screenshot_pending: bool
        @param screenshot_pending: True if the screenshot is in a pending state
            on the server.
        @type screenshot_list_window: gtk.Window
        @param screenshot_list_window: A reference to the screenshot-list
            window.
        @type _reference_thumbnail: tuple(2)|None
        @param reference_thumbnail: A byte-string and format-code containing
            the reference screenshot in thumbnail form.
        @type _reference_screenshot: tuple(2)|None
        @param _reference_screenshot: A byte-string and format-code containing
            the reference screenshot.
        """
        self._session = session
        self._screenshot_name = screenshot_name
        self._captured_screenshot = captured_screenshot
        self._screenshot_details = screenshot_details
        self._screenshot_pending = screenshot_pending
        self._screenshot_list_window = screenshot_list_window
        self._reference_thumbnail = _reference_thumbnail
        self._reference_screenshot = _reference_screenshot
        
        base.BaseWindow.__init__(self)
        while gtk.events_pending():
            gtk.main_iteration()
            
        context_id = self._sbr_status.get_context_id('screenshot')
        
        self._sbr_status.push(context_id, "Rendering captured screenshot...")
        try:
            (screenshot, format) = self._captured_screenshot
            screenshot.seek(0)
            pixbuf = quickshot.common.get_pixbuf_from_string(screenshot.getvalue(), format)
        except Exception, e:
            logger.critical("Unable to render captured screenshot: %(error)s" % {
             'error': str(e),
            })
            dialogue.error(self, (
             "Unable to display captured screenshot.",
             "This is incredibly bad. Quickshot cannot proceed.",
            ), recoverable=False)
            gtk.main_quit()
        else:
            self._img_new.set_from_pixbuf(pixbuf)
        finally:
            self._sbr_status.pop(context_id)
            
        self._sbr_status.push(context_id, "Downloading reference screenshot from server...")
        try:
            data = content_type = None
            if self._reference_screenshot:
                logger.debug("Loading cached reference screenshot")
                (data, content_type) = self._reference_screenshot
            else:
                logger.debug("Downloading reference screenshot")
                (data, content_type) = self._session.download_reference(
                 self._screenshot_name,
                )
        except quickshot.lib.server.NoScreenshotError, e:
            logger.info("No screenshot data found: %(url)s" % {
             'url': str(e),
            })
            container = self._img_reference.parent
            self._img_reference.destroy()
            label = gtk.Label("No reference screenshot available")
            container.add(label)
            label.show()
        except Exception, e:
            logger.warn("Unable to communicate with server: %(error)s" % {
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to display reference screenshot.",
             "The server could not be contacted.",
             "Error: " + str(e),
            ))
        else:
            self._sbr_status.push(context_id, "Rendering reference screenshot...")
            try:
                pixbuf = quickshot.common.get_pixbuf_from_string(data, content_type)
            except Exception, e:
                logger.warn("Unintelligible screenshot data received from server: %(error)s" % {
                 'error': str(e),
                })
                dialogue.warn(self, (
                 "Unable to display reference screenshot.",
                 "The file received from the server is corrupt.",
                ))
            else:
                self._reference_screenshot = (data, content_type)
                
                self._img_reference.set_from_pixbuf(pixbuf)
            finally:
                self._sbr_status.pop(context_id)
        finally:
            self._sbr_status.pop(context_id)
            
    def _build_content(self):
        """
        Loads the window structure from Glade.

        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        logger.debug("Loading screenshot_compare")
        
        builder = gtk.Builder()
        builder.add_from_file(quickshot.common.get_data_file('ui', 'screenshot_compare.ui'))
        
        self._btn_abort = builder.get_object('btn_abort')
        self._btn_recapture = builder.get_object('btn_recapture')
        self._btn_submit = builder.get_object('btn_submit')
        
        self._img_new = builder.get_object('img_new')
        self._img_reference = builder.get_object('img_reference')
        
        content = builder.get_object('box_main')
        self.set_title(content.parent.get_title())
        self.set_size_request(*content.parent.get_default_size())
        self.set_resizable(content.parent.get_resizable())
        self._sbr_status.set_has_resize_grip(self.get_resizable())
        self.set_icon(content.parent.get_icon())
        content.unparent()
        return content

    def _recapture(self, *args):
        """
        Returns to the screenshot-details stage, allowing the user to recapture
        the screenshot.

        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("Recapturing screenshot")
        
        self.disconnect(self._sig_destroy)
        self.destroy()
        screenshot_detail.ScreenshotDetailWindow(
         self._session,
         self._screenshot_name,
         self._screenshot_details,
         self._screenshot_pending,
         self._screenshot_list_window,
         _reference_thumbnail = self._reference_thumbnail,
         _reference_screenshot = self._reference_screenshot
        )

        #Dereference the buffers, just in case GTK takes a while.
        self._captured_screenshot = None
        self._reference_thumbnail = None
        self._reference_screenshot = None

    def _return_to_list(self, *args):
        """
        Returns to the screenshot-list stage

        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("Returning to screenshot list")
        
        self.hide()
        
        self._screenshot_list_window.show()
        
        #Dereference the buffers, just in case GTK takes a while.
        self._captured_screenshot = None
        self._reference_thumbnail = None
        self._reference_screenshot = None

        self.disconnect(self._sig_destroy)
        self.destroy()
        
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        self._sig_destroy = self.connect("destroy", self._return_to_list)
        
        self._btn_abort.connect("clicked", self._return_to_list)
        self._btn_recapture.connect("clicked", self._recapture)
        self._btn_submit.connect("clicked", self._upload)
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        self.set_position(gtk.WIN_POS_CENTER)
        
    def _upload(self, *args):
        """
        Uploads the captured screenshot to the server, trying repeatedly in the
        event of any unexpected errors, until the user aborts the process or
        the upload succeeds.

        Upon success, the user is returned to the screenshot list.

        @type *args: arg*
        @param *args: Not used.
        """
        contributor_details = quickshot.common.read_contributor_details()
        if not contributor_details:
            self._show_contributor_details()
            return
        else:
            logger.debug("Setting uploader credit values")
            self._session.set_uploader_details(
             contributor_details.get('e-mail'),
             contributor_details.get('name'),
             contributor_details.get('homepage'),
             contributor_details.get('publish')
            )

        logger.info("Uploading screenshot " + self._screenshot_name)
        context_id = self._sbr_status.get_context_id('screenshot')
        while True:
            self._sbr_status.push(context_id, "Uploading screenshot...")
            self.set_sensitive(False)
            try:
                while gtk.events_pending():
                    gtk.main_iteration()
                    
                (screenshot, format) = self._captured_screenshot
                screenshot.seek(0)
                self._session.upload_capture(screenshot, self._screenshot_name)
            except Exception, e:
                logger.error("Unable to upload screenshot: %(error)s" % {
                 'error': str(e),
                })
                if dialogue.question(self, (
                 "Unable to upload screenshot to server.",
                 "Error: " + str(e),
                 "Would you like to try again?",
                )) == gtk.RESPONSE_NO:
                    break
            else:
                logger.info("Uploaded screenshot " + self._screenshot_name)
                self._return_to_list()
                break
            finally:
                self.set_sensitive(True)
                self._sbr_status.pop(context_id)
                
