# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's
 screenshot-detail/pre-capture window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging
import subprocess
import time

import gobject
import gtk

import quickshot.common
import quickshot.lib.capture
import quickshot.lib.server

import base
import screenshot_compare
import dialogue

logger = logging.getLogger('quickshot.gui.screenshot_detail')

_SCREENSHOT_DELAY = 3000

class ScreenshotDetailWindow(base.BaseWindow):
    """
    A window used to brief the user on the details of the screenshot to be
    captured, presenting a reference sample, if available.
    """
    _session = None #: A L{quickshot.lib.server._Session} that manages communication with the server.
    _screenshot = None #: The system-name of the screenshot being worked on.
    _screenshot_pending = None #: True if the screenshot is in a pending state on the server.
    _screenshot_details = None #: Details about the screenshot to be captured.
    _screenshot_list_window = None #: A reference to the screenshot-list window.
    _reference_thumbnail = None #: A byte-string and format-code containing the reference screenshot in thumbnail form.
    _reference_screenshot = None #: A byte-string and format-code containing the reference screenshot.
    _btn_previous = None #: A button used to return to the screenshot-list window.
    _btn_capture = None #: A button used to proceed with capturing the screenshot.
    _btn_reference = None #: A button used to view a full-size version of the reference screenshot.
    _sig_destroy = None #: A signal-binding used to handle the window's on-destroy event.
    
    def __init__(self,
     session, screenshot, screenshot_details, screenshot_pending, screenshot_list_window,
     _reference_thumbnail=None, _reference_screenshot=None
    ):
        """
        Creates a screenshot-details window, used to allow the user to review
        screenshot details prior to capturing.

        The reference screenshot thumbnail, if not previously cached as part of
        a recapture loop, is downloaded from the server, if it is available.
        
        @type session: L{quickshot.lib.server._Session}
        @param session: A session object for managing communication with the
            project server.
        @type screenshot: basestring
        @param screenshot: The system-name of the screenshot.
        @type screenshot_details: dict
        @param screenshot_details: Details about the screenshot that was
            captured.
        @type screenshot_pending: bool
        @param screenshot_pending: True if the screenshot is in a pending state
            on the server.
        @type screenshot_list_window: gtk.Window
        @param screenshot_list_window: A reference to the screenshot-list
            window.
        @type _reference_thumbnail: tuple(2)|None
        @param reference_thumbnail: A byte-string and format-code containing
            the reference screenshot in thumbnail form.
        @type _reference_screenshot: tuple(2)|None
        @param _reference_screenshot: A byte-string and format-code containing
            the reference screenshot.
        """
        self._session = session
        self._screenshot = screenshot
        self._screenshot_pending = screenshot_pending
        self._screenshot_details = screenshot_details
        self._screenshot_list_window = screenshot_list_window
        self._reference_thumbnail = _reference_thumbnail
        self._reference_screenshot = _reference_screenshot
        
        base.BaseWindow.__init__(self)
        while gtk.events_pending():
            gtk.main_iteration()

        context_id = self._sbr_status.get_context_id('screenshot')
        self._sbr_status.push(context_id, "Downloading reference screenshot from server...")
        try:
            data = content_type = None
            if self._reference_thumbnail:
                logger.debug("Loading cached reference screenshot thumbnail")
                (data, content_type) = self._reference_thumbnail
            else:
                logger.debug("Downloading reference screenshot thumbnail")
                (data, content_type) = self._session.download_reference(
                 self._screenshot, thumbnail=(300, 300)
                )
        except quickshot.lib.server.NoScreenshotError, e:
            logger.debug("No screenshot data found: %(url)s" % {
             'url': str(e),
            })
        except Exception, e:
            logger.warn("Unable to communicate with server: %(error)s" % {
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to display reference screenshot.",
             "The server could not be contacted.",
             "Error: " + str(e),
            ))
        else:
            self._sbr_status.push(context_id, "Rendering reference screenshot...")
            try:
                pixbuf = quickshot.common.get_pixbuf_from_string(data, content_type)
            except Exception, e:
                logger.warn("Unintelligible screenshot data received from server: %(error)s" % {
                 'error': str(e),
                })
                dialogue.warn(self, (
                 "Unable to display reference screenshot.",
                 "The file received from the server is corrupt.",
                ))
            else:
                self._reference_thumbnail = (data, content_type)
                
                image = gtk.Image()
                image.set_from_pixbuf(pixbuf)
                
                self._btn_reference.set_label('')
                for child in self._btn_reference.get_children():
                    self._btn_reference.remove(child)
                self._btn_reference.add(image)
                self._btn_reference.set_image(image)
                self._btn_reference.set_sensitive(True)
                image.show()
            finally:
                self._sbr_status.pop(context_id)
        finally:
            self._sbr_status.pop(context_id)
            
    def _build_content(self):
        """
        Loads the window structure from Glade, then sets static values in the
        labels.

        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        logger.debug("Loading screenshot_detail")
        
        builder = gtk.Builder()
        builder.add_from_file(quickshot.common.get_data_file('ui', 'screenshot_detail.ui'))
        
        self._btn_previous = builder.get_object('btn_previous')
        self._btn_capture = builder.get_object('btn_capture')

        self._btn_reference = builder.get_object('btn_reference')
        self._btn_reference.set_uri(self._session.get_reference_url(self._screenshot))
        
        if self._screenshot_pending:
            builder.get_object('btn_latest').set_uri(self._session.get_latest_url(self._screenshot))
        else:
            builder.get_object('btn_latest').destroy()

        command = self._screenshot_details.get('command')
        if command:
            command = ' '.join([('"' + arg + '"') for arg in command])
            builder.get_object('lbl_command_value').set_text(command)
        else:
            builder.get_object('box_command').destroy()

        lbl_name = builder.get_object('lbl_name')
        lbl_name.set_markup(lbl_name.get_label() % {
         'name': '<i>' + self._screenshot_details['summary'] + '</i>',
        })

        self._render_steps(builder.get_object('lbl_steps_value'))
        
        content = builder.get_object('box_main')
        self.set_title(content.parent.get_title())
        self.set_size_request(*content.parent.get_default_size())
        self.set_resizable(content.parent.get_resizable())
        self._sbr_status.set_has_resize_grip(self.get_resizable())
        self.set_icon(content.parent.get_icon())
        content.unparent()
        return content

    def _capture_screenshot(self, *args):
        """
        Runs the command associated with the screenshot, if any, and, if
        successful, prompts the user to take the screenshot on a delay of
        L{_SCREENSHOT_DELAY} seconds.

        If the command failed, the detail view is restored.
        
        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("Capturing screenshot: " + repr(self._screenshot_details))
        
        self.hide()

        try:
            (success, active_window) = self._run_command()
            if success:
                seconds = int(round(_SCREENSHOT_DELAY / 1000.0))
                dialogue.info(self, (
                 "After you click OK, the screenshot will be captured in %(count)i second%(plural)s." % {
                  'count': seconds,
                  'plural': (not seconds == 1 and 's') or '',
                 },
                 "Immediately after clicking OK, please ensure the needed window has focus and is not obscured.",
                ))
                gobject.timeout_add(_SCREENSHOT_DELAY, self._handle_capture, active_window)
            else:
                self.show()
        except Exception, e:
            logger.error("Unable to process command: %(error)s" % {
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to process %(command)s" % {
              'command': repr(self._screenshot_details.get('command')),
             },
             "Error: " + str(e),
            ))
            self.show()
            
    def _handle_capture(self, active_window):
        """
        Captures the currently active window, activating the last-known-active
        window if the user did not explicitly give focus to a target.
        Alternatively, captures the full desktop.

        If GTK reports, though its own rendering of widgets, that a RTL
        language is in use, all relevant co-ordinates are automatically
        flipped.

        Upon a successful capture, the screenshot-comparison window is shown.

        If the capture fails, the detail view is shown.

        @type *args: arg*
        @param *args: Not used.
        """
        try:
            screenshot = format = None
            sub_rect_invert_x = self._screenshot_details.get('sub-rect-invert-x')
            if self._btn_reference.get_direction() == gtk.TEXT_DIR_RTL:
                sub_rect_invert_x = not sub_rect_invert_x
                
            if self._screenshot_details.get('fullscreen'):
                (screenshot, format) = quickshot.lib.capture.capture_screen(
                 sub_rect=self._screenshot_details.get('sub-rect'),
                 sub_rect_invert_x=sub_rect_invert_x,
                 sub_rect_invert_y=self._screenshot_details.get('sub-rect-invert-y')
                )
            else:
                (screenshot, format) = quickshot.lib.capture.capture_window(
                 sub_rect=self._screenshot_details.get('sub-rect'),
                 sub_rect_invert_x=sub_rect_invert_x,
                 sub_rect_invert_y=self._screenshot_details.get('sub-rect-invert-y'),
                 window=active_window
                )
        except Exception, e:
            logger.error("Unable to capture screenshot: %(error)s" % {
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to capture screenshot.",
             "Error: " + str(e),
            ))
            
            self.show()
        else:
            self.hide()
            
            screenshot_compare.ScreenshotCompareWindow(
             self._session,
             self._screenshot,
             (screenshot, format),
             self._screenshot_details,
             self._screenshot_pending,
             self._screenshot_list_window,
             _reference_thumbnail = self._reference_thumbnail,
             _reference_screenshot = self._reference_screenshot
            )

            #Dereference the buffers, just in case GTK takes a while.
            self._reference_thumbnail = None
            self._reference_screenshot = None

            self.disconnect(self._sig_destroy)
            self.destroy()
        finally:
            return False #Kill the timer.
            
    def _render_steps(self, label):
        """
        Renders steps with markup to increase readability.

        @type label: gtk.Label
        @param label: The label in which steps are to be rendered.
        """
        text = []
        list_order = 0
        for (format, step) in self._screenshot_details['steps']:
            if format == quickshot.lib.server.STEP_ORDERED:
                list_order += 1
                text.append('<b>%(i)i.</b> %(step)s' % {
                 'i': list_order,
                 'step': step,
                })
            else:
                list_order = 0
                if format == quickshot.lib.server.STEP_UNORDERED:
                    text.append('<b>*</b> ' + step)
                elif format == quickshot.lib.server.STEP_NOTE:
                    text.append('<b>Note</b>')
                    text.append('\t' + step)
                elif format == quickshot.lib.server.STEP_WARNING:
                    text.append('<b>Warning</b>')
                    text.append('\t' + step)
                else:
                    text.append(step)
        label.set_markup('\n'.join(text))
        
    def _return_to_list(self, *args):
        """
        Returns to the screenshot-list stage

        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("Returning to screenshot list")
        
        self.hide()
        
        self._screenshot_list_window.show()

        #Dereference the buffers, just in case GTK takes a while.
        self._reference_thumbnail = None
        self._reference_screenshot = None

        self.disconnect(self._sig_destroy)
        self.destroy()

    def _run_command(self):
        """
        Runs the command associated with a screenshot, if one exists.

        This will spawn a new process that may or may not itself become
        attached to an XID.

        In the event that the PID does become associated with an XID, the
        process is deemed to have run successfully, and the XID, if one exists,
        may be maximized, if necessary for the screenshot.

        If the PID terminates, it may be a case similar to Nautilus or Firefox,
        whereby the effect is forming a new child element of an already-running
        process. This is handled in the case of an expected maximized window
        with a named workaround process.

        If no PID-to-XID association is formed within ten seconds, the user
        will be asked if they would like to continue waiting. Opting to not
        wait will B{NOT} interrupt the process.

        If the process terminates unexpectedly, the user will be informed.

        @rtype: tuple(2)
        @return: True if the operation succeeded and the C{wnck.Window} of the
            activated window, if any.
        """
        command = self._screenshot_details.get('command')
        active_window = None
        
        if command:
            logger.info("Spawning sub-process: %(command)s" % {
             'command': repr(command),
            })
            popen = subprocess.Popen(command)

            timeout = 10.0
            process_terminated = False
            while True:
                time.sleep(0.2)
                timeout -= 0.2
                
                if not popen.poll() is None:
                    logger.debug("Process terminated")
                    process_terminated = True
                    break

                if quickshot.lib.capture.has_window(popen.pid):
                    logger.debug("Process spawned window")
                    break

                if timeout <= 0.0:
                    logger.debug("Timed out while waiting for process to spawn window")
                    if dialogue.question(self, (
                     "The issued command has neither terminated, nor spawned a window of its own.",
                     "The program may just be taking a while to start, or it may have started silently.",
                     "Would you like to continue waiting?",
                    )) == gtk.RESPONSE_NO:
                        logger.debug("User declined to continue waiting")
                        if dialogue.question(self, (
                         "Has the window opened on its own?",
                        )) == gtk.RESPONSE_NO:
                            logger.debug("User said no window spawned")
                            return (False, active_window)
                        else:
                            logger.debug("User said window spawned")
                            break
                    timeout = 10.0
            time.sleep(0.5)
            
            if not self._screenshot_details.get('maximize'):
                if quickshot.lib.capture.activate_window(popen.pid):
                    logger.debug("Activated window")
                    active_window = quickshot.lib.capture.get_active_window()
                else:
                    logger.debug("Unable to activate window")
                    dialogue.warn(self, (
                     "Unable to activate window.",
                     "You will need to manually give focus to the needed window.",
                    ))
            else:
                maximize_workaround = self._screenshot_details.get('maximize-workaround')
                if not maximize_workaround:
                    if process_terminated:
                        logger.debug("Process termination was unexpected")
                        if dialogue.warn(self, (
                         "The command seems to have failed, though this may be a false positive.",
                         "If you see the window you need to capture, just maximize it manually.",
                         "If not, click 'Stop' to abort the capture.",
                        ), abortable=True) == gtk.RESPONSE_NO:
                            logger.debug("User aborted capture")
                            return (False, active_window)
                    else:
                        if not quickshot.lib.capture.maximize_window(popen.pid):
                            logger.debug("Unable to maximize window")
                            dialogue.warn(self, (
                             "Unable to maximize window.",
                             "You will need to manually maximize the needed window.",
                            ))
                        else:
                            logger.debug("Window maximized")
                            active_window = quickshot.lib.capture.get_active_window()
                else:
                    logger.debug("Determining PIDs in maximize-workaround mode")
                    popen_pids = subprocess.Popen(('pidof', maximize_workaround,), stdout=subprocess.PIPE)
                    popen_pids.wait()

                    maximized = False
                    for pid in [int(p) for p in popen_pids.stdout.read().strip().split(' ') if p.isdigit()]:
                        maximized = quickshot.lib.capture.maximize_window(pid) or maximized

                    if not maximized:
                        logger.debug("No windows maximized")
                        if dialogue.warn(self, (
                         "No candidate windows were found.",
                         "This may be a false positive; if you see the window you need to capture, just maximize it manually.",
                        ), abortable=True) == gtk.RESPONSE_NO:
                            logger.debug("User aborted capture")
                            return (False, active_window)
                    else:
                        logger.debug("Window maximized")
                        active_window = quickshot.lib.capture.get_active_window()
        return (True, active_window)
        
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        self._sig_destroy = self.connect("destroy", self._return_to_list)
        
        self._btn_previous.connect("clicked", self._return_to_list)
        self._btn_capture.connect("clicked", self._capture_screenshot)
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        self.set_position(gtk.WIN_POS_CENTER)

        self._btn_reference.set_sensitive(False)
        self._btn_reference.set_tooltip_text("View the full-size reference image")
        
