# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's
 screenshot-list window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging
import os

import gobject
import gtk

import quickshot.common

import base
import screenshot_detail
import user_details
import dialogue

logger = logging.getLogger('quickshot.gui.screenshot_list')

class ScreenshotListWindow(base.BaseWindow):
    """
    A window used to allow the user to decide which screenshot to capture next.
    """
    _session = None #: A L{quickshot.lib.server._Session} that manages communication with the server.
    _details = None #: A dictionary of project details.
    _btn_forward = None #: A button used to start capturing a screenshot.
    _btn_close = None #: A button used to quit Quickshot.
    _btn_reload = None #: A button used to reload the screenshot progress list.
    _btn_homepage = None #: A button used to access the project's homepage.
    _btn_quickshot = None #: A button used to access the project's Quickshot progress.
    _lst_interface_language = None #: A list of all available instruction-languages for the project.
    _lst_screenshots = None #: A list of all screenshots that have yet to be accepted.

    def __init__(self, session, details):
        """
        Creates a screenshot-list window.

        If the user has not yet defined their contributor details, they are
        prompted to enter them now.

        @type session: L{quickshot.lib.server._Session}
        @param session: A session object for managing communication with the
            project server.
        @type details: dict
        @param details: A collection of project details.
        """
        self._session = session
        self._details = details
        
        base.BaseWindow.__init__(self)
        
        if not quickshot.common.read_contributor_details():
            self._show_contributor_details()
            
    def show(self):
        """
        Causes windows to reload their screenshot list every time they are
        shown.
        """
        logger.debug("Displaying screenshot_list")
        self._populate_screenshots()
        base.BaseWindow.show(self)
        
    def _build_content(self):
        """
        Loads the window structure from Glade, then sets static values in the
        labels.
        
        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        logger.debug("Loading screenshot_list")
        
        builder = gtk.Builder()
        builder.add_from_file(quickshot.common.get_data_file('ui', 'screenshot_list.ui'))
        
        lbl_summary = builder.get_object('lbl_summary')
        lbl_summary.set_markup(lbl_summary.get_text() % {
         'version_name': '<i>' + self._details['meta']['version']['friendly-name'] + '</i>',
        })

        builder.get_object('lbl_language_value').set_text(
         os.getenv('LANG')[:os.getenv('LANG').find('.')]
        )
        
        self._btn_close = builder.get_object('btn_close')
        self._btn_forward = builder.get_object('btn_forward')
        self._btn_homepage = builder.get_object('btn_homepage')
        self._btn_quickshot = builder.get_object('btn_quickshot')
        self._btn_reload = builder.get_object('btn_reload')
        self._lst_interface_language = builder.get_object('lst_interface_language')
        self._lst_screenshots = builder.get_object('lst_screenshots')
        
        content = builder.get_object('box_main')
        self.set_title(content.parent.get_title())
        self.set_size_request(*content.parent.get_default_size())
        self.set_resizable(content.parent.get_resizable())
        self._sbr_status.set_has_resize_grip(self.get_resizable())
        self.set_icon(content.parent.get_icon())
        content.unparent()
        return content

    def _get_selected_screenshot(self):
        """
        Provides the system-name and pending state of the currently selected
        screenshot.

        @rtype: tuple(2) | None
        @return: The currently selected screenshot's name and True if the
            screenshot is in the pending state; None if no screenshot is
            selected.
        """
        (model, iter) = self._lst_screenshots.get_selection().get_selected()
        if iter:
            (short_name, pending) = model.get(iter, 2, 1)
            return (short_name, pending == 'pending')
        return None

    def _interface_language_changed(self, combobox):
        """
        Updates the screenshot list and server-session whenever the interface
        language changes.

        @type combobox: gtk.ComboBox
        @param combobox: The interface language combobox.
        """
        selected = combobox.get_active_iter()
        model = combobox.get_model()
        interface_language = model.get_value(selected, 1)

        self._session.set_interface_language(interface_language)
        self._populate_screenshots()

    def _populate_interface_languages(self):
        """
        Populates the interface language combobox with all languages supported
        by the current project, plus the fallback 'Default' value.
        """
        logger.debug("Displaying interface languages")
        
        languages = gtk.ListStore(
         gobject.TYPE_STRING,
         gobject.TYPE_STRING
        )

        languages.append(('Default', ''))
        for language in self._details['meta']['languages']:
            languages.append((language, language))
            
        self._lst_interface_language.set_model(languages)
        cell = gtk.CellRendererText()
        self._lst_interface_language.pack_start(cell, True)
        self._lst_interface_language.add_attribute(cell, 'text', 0)
        self._lst_interface_language.set_active(0)
        
    def _populate_screenshots(self, *args):
        """
        Queries the server for a list of all non-accepted screenshots, then
        renders them in the main list.

        Those in the pending state are heavily shaded, and all listings are
        shaded in alternating tones for readability.

        Upon completion, the first non-pending screenshot is selected.

        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("Pulling screenshot list from server")
        
        while gtk.events_pending():
            gtk.main_iteration()
            
        screenshots = gtk.ListStore(
         gobject.TYPE_STRING,
         gobject.TYPE_STRING,
         gobject.TYPE_STRING,
         gobject.TYPE_STRING
        )
        
        context_id = self._sbr_status.get_context_id('screenshots')
        self._sbr_status.push(context_id, "Downloading screenshot list from server...")
        first_needed = None
        try:
            (
             (success, summary, details),
             (master, needed, pending, accepted),
            ) = self._session.query_progress()
            if not success:
                raise Exception("%(summary)s: %(details)s" % {
                 'summary': summary,
                 'details': details,
                })
        except Exception, e:
            logger.warn("Unable to download screenshot list: %(error)s" % {
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to download screenshot list from server.",
             "Click 'Reload' to try again.",
             "Error: " + str(e),
            ))
        else:
            logger.debug("Displaying screenshot list")
            
            needed = [(n, (name, 'needed')) for (n, name) in needed]
            pending = [(p, (name, 'pending')) for (p, name) in pending]
            pool = dict(needed + pending)

            position = 0
            for short_name in master:
                entry = pool.get(short_name)
                if entry:
                    (name, status) = entry
                    screenshot_needed = status == 'needed'
                    brightness = 0xff
                    if not screenshot_needed:
                        brightness -= 0x30
                    if position % 2:
                        brightness -= 0x10
                        
                    screenshots.append((
                     name,
                     status,
                     short_name,
                     '#%(brightness)02x%(brightness)02x%(brightness)02x' % {
                      'brightness': brightness,
                     },
                    ))
                    
                    if screenshot_needed and first_needed is None:
                        first_needed = position
                    position += 1
        finally:
            self._sbr_status.pop(context_id)
            
        self._lst_screenshots.set_model(screenshots)
        
        if not first_needed is None:
            active_iter = screenshots.get_iter_first()
            for i in xrange(first_needed):
                active_iter = screenshots.iter_next(active_iter)
            self._lst_screenshots.get_selection().select_iter(active_iter)
            self._btn_forward.set_sensitive(True)

    def _screenshot_selected(self, treeview, button):
        """
        Enables the screenshot-capture button whenever a screenshot is
        selected.

        @type treeview: gtk.TreeView
        @param treeview: The screenshot list.
        @type button: gtk.Button
        @param button: The screenshot-capture button.
        """
        logger.debug("Screenshot selection changed")
        selection = self._get_selected_screenshot()
        button.set_sensitive(bool(selection))
        
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        self.connect("destroy", gtk.main_quit)
        
        self._btn_close.connect("clicked", gtk.main_quit)
        self._btn_forward.connect("clicked", self._show_screenshot_details)
        self._btn_reload.connect("clicked", self._populate_screenshots)
        
        self._lst_screenshots.connect("cursor-changed", self._screenshot_selected, self._btn_forward)
        self._lst_screenshots.connect("row-activated", self._show_screenshot_details)
        self._lst_interface_language.connect("changed", self._interface_language_changed)
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        self.set_position(gtk.WIN_POS_CENTER)
        
        self._btn_forward.set_sensitive(False)
        
        project_homepage = self._details['meta']['version'].get('homepage') or self._details['meta']['project'].get('homepage')
        if project_homepage:
            self._btn_homepage.set_uri(project_homepage)
        else:
            self._btn_homepage.destroy()
            
        quickshot_progress = self._session.get_address()
        if quickshot_progress:
            self._btn_quickshot.set_uri(quickshot_progress)
        else:
            if not project_homepage:
                self._btn_quickshot.parent.parent.destroy()
            else:
                self._btn_quickshot.destroy()

        self._populate_interface_languages()
        
        cell = gtk.CellRendererText()
        cell.set_property('background-set' , True)
        column = gtk.TreeViewColumn("Name")
        column.pack_start(cell, True)
        column.set_attributes(cell, text=0, background=3)
        self._lst_screenshots.append_column(column)
        
        cell = gtk.CellRendererText()
        cell.set_property('background-set' , True)
        column = gtk.TreeViewColumn("Status")
        column.pack_start(cell, True)
        column.set_attributes(cell, text=1, background=3)
        self._lst_screenshots.append_column(column)
        
    def _show_screenshot_details(self, *args):
        """
        Pulls details for the selected screenshot, if any, from the server, and
        presents the user with the screenshot-details prompt for capturing.

        @type *args: arg*
        @param *args: Not used.
        """
        screenshot = self._get_selected_screenshot()
        if screenshot:
            (screenshot, pending) = screenshot
            logger.debug("Pulling details for " + screenshot + " from server")
            
            context_id = self._sbr_status.get_context_id('screenshot')
            self._sbr_status.push(context_id, "Downloading screenshot details from server...")
            try:
                (
                 (success, summary, details),
                 screenshot_details,
                ) = self._session.query_details(screenshot)
                if not success:
                    raise Exception("%(summary)s: %(details)s" % {
                     'summary': summary,
                     'details': details,
                    })
            except Exception, e:
                logger.warn("Unable to download screenshot details: %(error)s" % {
                 'error': str(e),
                })
                dialogue.warn(self, (
                 "Unable to download screenshot details from server.",
                 "Error: " + str(e),
                ))
            else:
                self.hide()
                screenshot_detail.ScreenshotDetailWindow(
                 self._session,
                 screenshot,
                 screenshot_details,
                 pending,
                 self
                )
            finally:
                self._sbr_status.pop(context_id)
                
