# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's user-account window.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging
import os
import pwd

import gtk

import quickshot.common
import quickshot.lib.account

import base
import graphics_change
import dialogue

logger = logging.getLogger('quickshot.gui.user_account')

class UserAccountWindow(base.BaseWindow):
    """
    A window used to guide the user through the process of defining a user
    account for projects that require one.
    """
    _details = None #: A dictionary of project details.
    _username = None #: The username required by the project.
    _display_name = None #: The display-name required by the project.
    _btn_create = None #: A button used to engage the account-modification process.
    _btn_quit = None #: A button used to quit Quickshot.
    _txt_password = None #: An input for the account's password.
    _txt_confirm = None #: An input to confirm the account's password.

    def __init__(self, session, details):
        """
        Either creates a user-account window or bypasses this step, immediately
        loading the resolution-change window.

        The decision is made based on whether the current user meets all
        project-required criteria.

        @type session: L{quickshot.lib.server._Session}
        @param session: A session object for managing communication with the
            project server.
        @type details: dict
        @param details: A collection of project details.
        """
        self._details = details
        self._username = details['config'].get('username') or ''
        self._display_name = details['config'].get('display-name') or ''
        
        username = os.environ.get('LOGNAME')
        if not self._username or (self._username == username and self._display_name in pwd.getpwnam(username).pw_gecos):
            logger.debug("User account already compatible with project")
            graphics_change.GraphicsChangeWindow(session, details)
            return

        account_status = quickshot.lib.account.validate_account(self._username, self._display_name)
        if account_status == quickshot.lib.account.ACCOUNT_EXISTS:
            logger.debug("Project account already exists")
            self._show_logout_prompt()
            return
            
        base.BaseWindow.__init__(self)
        
    def _build_content(self):
        """
        Loads the window structure from Glade, then sets static values in the
        labels.

        @rtype: gtk.Widget
        @return: The widget to push into the heart of the window.
        """
        logger.debug("Loading user_account")
        
        builder = gtk.Builder()
        builder.add_from_file(quickshot.common.get_data_file('ui', 'user_account.ui'))

        lbl_summary = builder.get_object('lbl_summary')
        lbl_summary.set_markup(lbl_summary.get_text() % {
         'project_version': '<i>' + self._details['meta']['version']['friendly-name'] + '</i>',
        })
        
        lbl_username = builder.get_object('lbl_username')
        lbl_username.set_markup(lbl_username.get_text() % {
         'username': '<i>' + self._username + '</i>',
        })
        
        lbl_display_name = builder.get_object('lbl_display_name')
        lbl_display_name.set_markup(lbl_display_name.get_text() % {
         'display_name': '<i>' + self._display_name + '</i>',
        })
        
        self._btn_quit = builder.get_object('btn_quit')
        self._btn_create = builder.get_object('btn_create')
        self._txt_password = builder.get_object('txt_password')
        self._txt_confirm = builder.get_object('txt_confirm')
        
        content = builder.get_object('box_main')
        self.set_title(content.parent.get_title())
        self.set_size_request(*content.parent.get_default_size())
        self.set_resizable(content.parent.get_resizable())
        self._sbr_status.set_has_resize_grip(self.get_resizable())
        self.set_icon(content.parent.get_icon())
        content.unparent()
        return content
        
    def _create_account(self, *args):
        """
        Modifies the host's user account database, setting proper account
        details for the active project.

        Post-modification, the database is checked to make sure the update
        succeeded.
        
        @type *args: arg*
        @param *args: Not used.
        """
        logger.debug("User granted permission to modify account details")

        context_id = self._sbr_status.get_context_id('modify accounts')
        self._sbr_status.push(context_id, "Defining local user account data...")
        try:
            if not quickshot.lib.account.set_user_account(
             self._username,
             self._display_name,
             self._txt_password.get_text()
            ):
                logger.warn("Unable to issue user account update command")
                dialogue.warn(self, (
                 "Unable to modify user account.",
                 "The required command could not be issued; you may not have sufficient permission.",
                ))
                return
        except Exception, e:
            logger.error("Unable to modify user account: %(error)s" % {
             'error': str(e),
            })
            dialogue.warn(self, (
             "Unable to modify user account.",
             "Error: " + str(e),
            ))
        else:
            if quickshot.lib.account.validate_account(self._username, self._display_name) == quickshot.lib.account.ACCOUNT_EXISTS:
                self._show_logout_prompt()
                self.hide()
            else:
                logger.error("Unable to update user account database")
                dialogue.warn(self, (
                 "Unable to modify user account.",
                 "The accounts database could not be updated.",
                ))
        finally:
            self._sbr_status.pop(context_id)
            
    def _password_changed(self, password, confirm, button):
        """
        Enables the account-modification button if both password fields match.

        @type password: gtk.Entry
        @param password: One of the two password fields.
        @type confirm: gtk.Entry
        @param confirm: One of the two password fields.
        @type button: gtk.Button
        @param button: The button to be toggled.
        """
        button.set_sensitive(bool(password.get_text() and password.get_text() == confirm.get_text()))
        
    def _set_bindings(self):
        """
        Connects event-signals to handlers.
        """
        self.connect("destroy", gtk.main_quit)
        
        self._txt_password.connect("changed", self._password_changed, self._txt_confirm, self._btn_create)
        self._txt_confirm.connect("changed", self._password_changed, self._txt_password, self._btn_create)
        
        self._btn_quit.connect("clicked", gtk.main_quit)
        self._btn_create.connect("clicked", self._create_account)
        
    def _set_properties(self):
        """
        Sets display-properties and widget-states.
        """
        self.set_position(gtk.WIN_POS_CENTER)
        
        self._btn_create.set_sensitive(False)
        
    def _show_logout_prompt(self):
        """
        Prompts the user to log out, terminating Quickshot upon completion,
        regardless of the user's choice.
        """
        if dialogue.question(self, (
         "%(project_version)s requires you to log in as %(username)s before capturing screenshots." % {
          'project_version': self._details['meta']['version']['friendly-name'],
          'username': self._username,
         },
         "Would you like to log out now?"
        )) == gtk.RESPONSE_YES:
            logger.debug("User chose to log out")
            try:
                quickshot.lib.account.logout()
            except Exception, e:
                logger.warn("Unable to log out")
                dialogue.warn(self, (
                 "Unable to log out automatically.",
                 "Please log out manually when you are ready to continue."
                ))
        gtk.main_quit()
        
