# -*- coding: utf-8 -*-
"""
Overview
=======
 Handles the logic associated with the Quickshot client's user-details window,
 used to credit uploaders.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import logging
import re

import gtk

import quickshot.common

logger = logging.getLogger('quickshot.gui.user_details')

def UserDetailsWindow(parent):
    """
    Sets up and shows the user-details-window.

    @type parent: gtk.Window
    @param parent: The window to use as a parent.

    @rtype: gtk.Window
    @return: The user-details-window.
    """
    logger.debug("Loading user_details")
    
    builder = gtk.Builder()
    builder.add_from_file(quickshot.common.get_data_file('ui', 'user_details.ui'))
    
    user_details_window = builder.get_object('win_user_details')
    user_details_window.set_transient_for(parent)

    txt_email = builder.get_object('txt_email')
    txt_name = builder.get_object('txt_name')
    txt_homepage = builder.get_object('txt_homepage')
    chk_publish = builder.get_object('chk_publish')

    contributor_details = quickshot.common.read_contributor_details()
    if contributor_details:
        txt_email.set_text(contributor_details.get('e-mail') or '')
        txt_name.set_text(contributor_details.get('name') or '')
        txt_homepage.set_text(contributor_details.get('homepage') or '')
        chk_publish.set_active(contributor_details.get('publish') or False)
        
    btn_update = builder.get_object('btn_update')
    btn_update.connect("clicked", _write_details,
     user_details_window, txt_email, txt_name, txt_homepage, chk_publish
    )
    
    txt_email.connect("changed", _email_changed, btn_update)
    _email_changed(txt_email, btn_update)

    user_details_window.connect("destroy", _write_details,
     user_details_window, txt_email, txt_name, txt_homepage, chk_publish
    )
    
    user_details_window.show()
    return user_details_window
    
def _email_changed(txt_email, btn_update):
    """
    Allows the user to save their details if the e-mail field is properly populated.

    @type txt_email: gtk.Entry
    @param txt_email: The e-mail field.
    @type btn_update: gtk.Button
    @param btn_update: The update button.
    """
    btn_update.set_sensitive(bool(re.match("^\S+@\S+\.\S+$", txt_email.get_text()))) #Not a strict check.
    
def _write_details(button, window, txt_email, txt_name, txt_homepage, chk_publish):
    """
    Saves the user's details and destroys the window.

    @type button: gtk.Button
    @param button: Not used.
    @type window: gtk.Window
    @param window: The user-details window.
    @type txt_email: gtk.Entry
    @param txt_email: The e-mail field.
    @type txt_name: gtk.Entry
    @param txt_name: The name field.
    @type txt_homepage: gtk.Entry
    @param txt_homepage: The homepage field.
    @type chk_publish: gtk.CheckButton
    @param chk_publish: The publish-approval field.
    """
    logger.info("Writing contributor details")
    window.destroy()
    
    quickshot.common.write_contributor_details(
     txt_email.get_text() or 'anonymous@example.org', #Workaround for direct calls to destroy.
     txt_name.get_text(),
     txt_homepage.get_text(),
     chk_publish.get_active()
    )
    
