# -*- coding: utf-8 -*-
"""
Overview
=======
 Implements an interface for interacting with the host's user accounts,
 respecting access methods inherent to the active window manager.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import crypt
import os
import pwd
import random
import subprocess

import environment

_SALT_CHARACTERS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
"""
A sequence of legal characters that can be used to encrypt a password for
C{useradd}.
"""

ACCOUNT_EXISTS = 1 #: Indicates that the requested account exists.
ACCOUNT_NOT_FOUND = 2 #: Indicates that no such account exists.
ACCOUNT_NAME_INVALID = 3 #: Indicates that the accout exists, but the display name conflicts.

def _hash_password(password):
    """
    Creates a local-system-valid password-hash.

    @type password: basestring
    @param password: The plain-text password to be assigned to the user account.

    @rtype: basestring
    @return: The hashed password.
    """
    salt = random.choice(_SALT_CHARACTERS) + random.choice(_SALT_CHARACTERS)
    return crypt.crypt(password, salt)

def logout():
    """
    Issues the appropriate logout command for the user's environment.

    @raise environment.NoCommandError: The appropriate logout command could not
        be determined.
    """
    subprocess.call(environment.get_logout_command())
    
def set_user_account(username, display_name, password):
    """
    Adds or updates the host's account database, depending on the state of the
    specified account, using the appropriate privilege-escalation command.

    @type username: basestring
    @param username: The username of the user to be looked up.
    @type display_name: basestring
    @param display_name: The display-name of the user account, typically stored
        as comment meta-data.
    @type password: basestring
    @param password: The plain-text password to be assigned to the user account.

    @rtype: bool
    @return: True if the update succeeded.

    @raise environment.NoCommandError: The appropriate sudo command could not
        be determined.
    """
    account_status = validate_account(username, display_name)
    sudo_command = environment.get_elevate_privilages_command()
    
    if account_status == ACCOUNT_NOT_FOUND:
        return subprocess.call((
         sudo_command,
         "useradd --shell '/bin/bash' -m %(display_name)s-p '%(password)s' '%(username)s'" % {
          'display_name': (display_name and "-c '" + display_name +"' ") or '',
          'password': _hash_password(password),
          'username': username,
         }
        )) == 0
    elif account_status == ACCOUNT_NAME_INVALID:
        return subprocess.call((
         sudo_command,
         "usermod %(display_name)s-p '%(password)s' '%(username)s'" % {
          'display_name': (display_name and "-c '" + display_name +"' ") or '',
          'password': _hash_password(password),
          'username': username,
         }
        )) == 0
        
def validate_account(username, display_name):
    """
    Determines the status of the requested account in the host's account
    database.

    @type username: basestring
    @param username: The username of the user to be looked up.
    @type display_name: basestring
    @param display_name: The display-name of the user account, typically stored
        as comment meta-data.

    @rtype: int
    @return: One of the C{ACCOUNT_*} module-level constants.
    """
    try:
        user_details = pwd.getpwnam(username)
    except KeyError, e:
        return ACCOUNT_NOT_FOUND
    else:
        if display_name in user_details.pw_gecos:
            return ACCOUNT_EXISTS
        else:
            return ACCOUNT_NAME_INVALID
            
