# -*- coding: utf-8 -*-
"""
Overview
=======
 Implements the logic needed to capture screenshots.

 Currently presumes access to GTK libraries, as does the rest of Quickshot.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import gc
import time
import cStringIO

import gtk
import wnck

def _capture(geometry, format, sub_rect, sub_rect_invert_x, sub_rect_invert_y):
    """
    Captures the specified region to a C{gtk.gdk.Pixbuf}, then writes it to
    C{file_path}, using the C{format} codec.
    
    @type geometry: sequence(4)
    @param left: The (left, top, width, height) bounding rectangle to be captured.
    @type format: basestring
    @param format: One of C{('png', 'jpeg',)}.
    @type sub_rect: tuple(4)|None
    @param sub_rect: A rectangle C{(left:int, top:int, width:int, height:int)}
        to extract from within the screen, relative to the origin, ignoring
        decorations.
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_x: True if the rectangle should be extracted
        relative to the right edge.
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_y: True if the rectangle should be extracted
        relative to the bottom edge.
    
    @rtype: file-like
    @return: The captured screenshot data, as a file-like object.
    """
    (left, top, width, height) = _compute_geometry(
     geometry, sub_rect, sub_rect_invert_x, sub_rect_invert_y
    )
    
    #Build the Pixbuf and fill it with the image from the screen.
    capture = gtk.gdk.Pixbuf.get_from_drawable(
     gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, width, height),
     gtk.gdk.get_default_root_window(),
     gtk.gdk.colormap_get_system(),
     left, top, 0, 0, width, height
    )
    
    #Set up an in-memory handler for the save-operation.
    output = cStringIO.StringIO()
    def _save(data, sink):
        sink.write(data)
        return True
        
    #Save the Pixbuf to disk.
    options = {}
    if format == 'jpeg':
        options['quality'] = 100
    capture.save_to_callback(_save, format, options=options, user_data=output)
    
    #Force removal of Pixbuf-consumed memory.
    del capture
    gc.collect()

    output.seek(0)
    return output

def _compute_geometry(geometry, sub_rect, sub_rect_invert_x, sub_rect_invert_y):
    """
    Computes the co-ordinates needed for extracting the interesting part of a
    2D plane.

    From C{geometry}, the base-geometry of the window to be captured, the
    C{sub-rect}, if supplied, is used to extract a sub-rectangle relative to
    the geometry of its parent.

    The invert flags, if set, will cause the sub-rectangle's offset to be
    computed relative to the opposide edge of the parent. For example, if the
    parent window is 200px from the left of the screen and its width is 400,
    an inverted sub-rectangle will, with a width of 100 and a left-edge of 0,
    cause a left-edge of 500 and a width of 100 to be returned. A sub-rectangle
    width a left-edge of 500 and a width of 200 should cause a left-edge of 200
    and a width of 100 to be returned, since the window's bounding geometry is
    at 200px on its left edge, meaning that anything past that point must be
    culled.
    
    @type geometry: sequence(4)
    @param left: The (left, top, width, height) bounding rectangle to be captured.
    @type sub_rect: tuple(4)|None
    @param sub_rect: A rectangle C{(left:int, top:int, width:int, height:int)}
        to extract from within the screen, relative to the origin, ignoring
        decorations.
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_x: True if the rectangle should be extracted
        relative to the right edge.
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_y: True if the rectangle should be extracted
        relative to the bottom edge.

    @rtype: tuple(4)
    @return: The (left, top, width, height) bounding rectangle to be captured,
        after adjusting for sub-rect slicing.
    """
    (base_left, base_top, base_width, base_height) = geometry
    left = top = width = height = None
    
    if sub_rect:
        (sub_left, sub_top, sub_width, sub_height) = sub_rect
        right = base_left + base_width
        bottom = base_top + base_height
        
        if not sub_rect_invert_x:
            left = min(base_left + sub_left, right - 1)
            width = min(sub_width, right - left)
        else:
            left = right - sub_left - sub_width - 1
            if left < base_left:
                width = sub_width + (left - base_left)
                left = base_left
            else:
                width = sub_width
            width = min(width, right - left)
            
        if not sub_rect_invert_y:
            top = min(base_top + sub_top, bottom - 1)
            height = min(sub_height, bottom - top)
        else:
            top = bottom - sub_top - sub_height - 1
            if top < base_top:
                height = sub_height + (top - base_top)
                top = base_top
            else:
                height = sub_height
            height = min(height, bottom - top)
    else:
        left = base_left
        top = base_top
        width = base_width
        height = base_height
        
    #Constrain capture area to visible surface
    root = gtk.gdk.screen_get_default()
    root_width = root.get_width()
    root_height = root.get_height()
    
    left = min(root_width - 2, max(0, left))
    top = min(root_height - 2, max(0, top))
    width = max(1, min(root_width - left, width))
    height = max(1, min(root_height - top, height))
    
    return (left, top, width, height)
    
def _update_window_manager_status():
    """
    Updates the representation of active windows, according to the window
    manager.
    """
    #Wait for GTK to process everything so the active window will definitely be
    #selected.
    while gtk.events_pending():
        gtk.main_iteration()
        
    #Force wnck to grab current data.
    wnck.screen_get_default().force_update()
    
def capture_screen(format='png', sub_rect=None, sub_rect_invert_x=False, sub_rect_invert_y=False):
    """
    Captures the active screen to a C{gtk.gdk.Pixbuf}, then writes it to
    C{file_path}, using the C{format} codec.
    
    @type format: basestring
    @param format: One of C{('png', 'jpeg',)}.
        
        Default: C{'png'}
    @type sub_rect: tuple(4)|None
    @param sub_rect: A rectangle C{(left:int, top:int, width:int, height:int)}
        to extract from within the screen, relative to the origin, ignoring
        decorations.
        
        Default: C{None}
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_x: True if the rectangle should be extracted
        relative to the right edge.

        Default: False
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_y: True if the rectangle should be extracted
        relative to the bottom edge.

        Default: False
    
    @rtype: tuple(2)
    @return: A file-like object containing the captured screenshot and the format of
        the captured screenshot, one of C{('png', 'jpeg',)}.
    """
    root = gtk.gdk.screen_get_default()
    geometry = (0, 0, root.get_width(), root.get_height())
    return (
     _capture(geometry, format, sub_rect, sub_rect_invert_x, sub_rect_invert_y),
     format,
    )
    
def capture_window(format='png', sub_rect=None, sub_rect_invert_x=False, sub_rect_invert_y=False, window=None):
    """
    Captures the active window to a C{gtk.gdk.Pixbuf}, then writes it to
    C{file_path}, using the C{format} codec.
    
    @type format: basestring
    @param format: One of C{('png', 'jpeg',)}.
        
        Default: C{'png'}
    @type sub_rect: tuple(4)|None
    @param sub_rect: A rectangle C{(left:int, top:int, width:int, height:int)}
        to extract from within the window, relative to its top-left corner,
        ignoring decorations.
        
        Default: C{None}
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_x: True if the rectangle should be extracted
        relative to the right edge.

        Default: False
    @type sub_rect_invert_x: bool
    @param sub_rect_invert_y: True if the rectangle should be extracted
        relative to the bottom edge.

        Default: False
    @type window: C{wnck.Window}
    @param window: A reference to the window to be activated, if an explicit
        reference is needed in the event that no active window is found.

        Default: None
    @rtype: tuple(2)
    @return: A file-like object containing the captured screenshot and the format of
        the captured screenshot, one of C{('png', 'jpeg',)}.
    
    @raise UnknownActiveWindowError: The active window could not be determined.
    """
    _update_window_manager_status()
    active_window = wnck.screen_get_default().get_active_window()
    if active_window:
        window = active_window
    elif window:
        window.activate(int(time.time()))
        
    if window:
        geometry = window.get_geometry()
        return (
         _capture(geometry, format, sub_rect, sub_rect_invert_x, sub_rect_invert_y),
         format,
        )
    else:
        raise UnknownActiveWindowError("Window manager did not supply an active window reference")
        
def activate_window(pid):
    """
    A convenience function that activates all windows running under the given
    PID.
    
    Note: This function does not guarantee window order and should be used only
    if a single window is spawned by a process.
    
    @type pid: int
    @param pid: The PID for which windows are to be maximized.

    @type: bool
    @return: True if windows were activated.
    """
    _update_window_manager_status()
    screen = wnck.screen_get_default()
    windows = [w for w in screen.get_windows() if w.get_pid() == pid]
    for window in windows:
        window.activate(int(time.time()))
    return bool(windows)

def get_active_window():
    """
    Gets a reference to the currently active window, if any.

    @rtype: C{wnck.Window}|None
    @return: The currently active window, or None if no active window exists.
    """
    _update_window_manager_status()
    return wnck.screen_get_default().get_active_window()
    
def has_window(pid):
    """
    Indicates whether a window has been associated with the given PID.
    
    @type pid: int
    @param pid: The PID for which to check.
    
    @rtype: bool
    @return: True if the PID has a window.
    """
    screen = wnck.screen_get_default()
    _update_window_manager_status()
    
    return bool([w for w in screen.get_windows() if w.get_pid() == pid])
    
def maximize_window(pid, give_focus=True):
    """
    A convenience function that maximizes all windows running under the given
    PID.
    
    Note: This function does not guarantee window order and should be used only
    if a single window is spawned by a process.
    
    @type pid: int
    @param pid: The PID for which windows are to be maximized.
    @type give_focus: bool
    @param give_focus: True if windows should be given focus.
        
        Default: C{True}

    @type: bool
    @return: True if windows were maximized.
    """
    _update_window_manager_status()
    screen = wnck.screen_get_default()
    
    windows = [w for w in screen.get_windows() if w.get_pid() == pid]
    for window in windows:
        window.maximize()
        if give_focus:
            window.activate(int(time.time()))
    return bool(windows)
    
class Error(Exception):
    """
    The base class from which all exceptions native to this module inherit.
    """
    pass
    
class UnknownActiveWindowError(Error):
    """
    Signifies that the active window could not be determined.
    """
    pass

