# -*- coding: utf-8 -*-
"""
Overview
=======
 Implements functions that manipulate the host's screen resolution.

Legal
=====
 All code, unless otherwise indicated, is original, and subject to the terms of
 the attached licensing agreement.
 
 Copyright (c) 2010
  Neil Tallim <flan@uguu.ca>
  Luke Jennings <ubuntujenkins@googlemail.com>
 
 Quickshot is free software: you can redistribute it and/or modify it 
 under the terms of the GNU General Public License version 3, as published 
 by the Free Software Foundation.
 
 Quickshot is distributed in the hope that it will be useful, but 
 WITHOUT ANY WARRANTY; without even the implied warranties of 
 MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License along 
 with Quickshot.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import subprocess

import gtk

def _build_change_command(x, y):
    """
    Assembles the appropriate command for changing the user's resolution.

    @type x: int
    @param x: The target X-resolution.
    @type y: int
    @param y: The target y-resolution.

    @rtype: tuple
    @return: The parameterised resolution-changing command.
    """
    driver = _detect_graphics_driver()
    if driver == 'nvidia':
        return ('disper', '-c', '-r', str(x) + 'x' + str(y),)
    return ('xrandr', '-s', str(x) + 'x' + str(y))

def _detect_graphics_driver():
    """
    Determines the driver currently in use on the host's system by looking at
    all modules currently loaded by the kernel.

    @rtype: str
    @return: The name of the active driver.
    """
    candidate_modules = (
     'nvidia',
     'nouveau',
     'radeon',
     'i915',
    )
    try:
        modules = set(os.listdir('/sys/module'))
        for module in candidate_modules:
            if module in modules:
                return module
    except Exception, e:
        pass
    return 'generic'
    
def get_resolution():
    """
    Provides the current screen resolution.

    @rtype: tuple(2)
    @return: The (X, Y) resolution of the screen.
    """
    return (gtk.gdk.screen_width(), gtk.gdk.screen_height())

def set_resolution(x, y):
    """
    Sets the current screen resolution.

    @type x: int
    @param x: The target X-resolution.
    @type y: int
    @param y: The target y-resolution.

    @raise ResolutionChangeError: The screen resolution could not be changed.
    """
    try:
        command = _build_change_command(x, y)
        subprocess.check_call(command)
    except subprocess.CalledProcessError, e:
        raise ResolutionChangeError("Unable to change resolution: %(error)s; command: %(command)s" % {
         'error': str(e),
         'command': repr(command),
        })
        
class Error(Exception):
    """
    The base class from which all exceptions native to this module inherit.
    """
    pass

class ResolutionChangeError(Error):
    """
    Signifies that the resolution was not changed.
    """
    pass
    
